#
#
# OK, here we perform the main experiments for the paper.
# The goal is to essentially perform as a makefile and
# recreate all of the experiments at once.
#
#
source('regression_lib.R')

boot_times <- 1000; 

#
# Calculate the subset index. Use the same for everything
#

x <- build_dataset('ensembl.fimo.only.-30000to+30000.tsv');
# Select 20% of the dataset to select the sigma
subset.ind <- sample(nrow(x), floor(0.2*nrow(x)));

# Global result arrays
rsq <- array();
rsq.l <- array();
rsq.u <- array();
rsqcount <- 0;

#
# 1. Run FIMO-only (just for R-squared bootstrap)
#
formula <- esc ~  E2f1 + Esrrb + Klf4 + Nanog + Oct4 + Smad1 + Sox2 + Stat3 + Tcfcp2l1 + Zfx + c.Myc + n.Myc;

x <- build_dataset('ensembl.fimo.only.-30000to+30000.tsv');
subset <- x[subset.ind,];
remainder <- x[-subset.ind,];

sigma <- find_best_sigma(subset,formula);

# Prepare data for bootstrap
y <- log(remainder$esc + sigma);
x.log <- remainder;
x.log[,3:14] <- normalize.quantiles(as.matrix(x.log[,3:14]));
x.log <- log(x.log[,c(3:14)] + sigma);
data <- cbind(y, x.log);
colnames(data)[1] <- 'esc';

# Run bootstrap
results <- boot(data=data, formula=formula, statistic=bs_rsq, R=boot_times);

bci <- boot.ci(results, type="perc", conf=0.95, index=1);
rsqcount <- rsqcount + 1;
rsq[rsqcount] <- results$t0;
rsq.l[rsqcount] <- bci$percent[,4];
rsq.u[rsqcount] <- bci$percent[,5];
names(rsq)[rsqcount] <- "FIMO TFAS";

plot_correlation_scatter(data, formula, "fimo.only.scatter");

#
# 2. Run FIMO + H3K4me3 (R-squared bootstrap, SVD, coeff)
#
formula <- esc ~  E2f1 + Esrrb + Klf4 + Nanog + Oct4 + Smad1 + Sox2 + Stat3 + Tcfcp2l1 + Zfx + c.Myc + n.Myc;
svd_formula <- esc ~ PC1 + PC2 + PC3 + PC4 + PC5 + PC6 + PC7 + PC8 + PC9 + PC10 + PC11 + PC12;

x <- build_dataset('ensembl.fimoplus.loglikelihood.-30000to+30000.tsv');
subset <- x[subset.ind,];
remainder <- x[-subset.ind,];

sigma <- find_best_sigma(subset, formula);

# Prepare data for bootstrap
y <- log(remainder$esc + sigma);
x.log <- remainder;
x.log[,3:14] <- normalize.quantiles(as.matrix(x.log[,3:14]));
x.log <- log(x.log[,c(3:14)] + sigma);
data <- cbind(y, x.log);
colnames(data)[1] <- 'esc';

# Run bootstraps
results <- boot(data=data, formula=formula, statistic=bs_rsq, R=boot_times);

bci <- boot.ci(results, type="perc", conf=0.95, index=1);
rsqcount <- rsqcount + 1;
rsq[rsqcount] <- results$t0;
rsq.l[rsqcount] <- bci$percent[,4];
rsq.u[rsqcount] <- bci$percent[,5];
names(rsq)[rsqcount] <- "FIMO+ TFAS";

results_tstat <- boot(data=data, formula=formula, statistic=bs_tstat, R=boot_times);
plot_bootstrap_barplot(results_tstat, "fimo+.h3k4me3.nosvd.tstat");

data.svd <- svd_data(data);

svd_results_rsq <- boot(data=data.svd, statistic=bs_ind_rsq, R=boot_times);
svd_results_tstat <- boot(data=data.svd, formula=svd_formula, statistic=bs_tstat, R=boot_times);

plot_bootstrap_barplot(svd_results_tstat, "fimo+.h3k4me3.svd.tstat");
plot_svd_bootstrap_barplot(svd_results_tstat, svd_results_rsq, results_tstat, "fimo+.h3k4me3.svd.tstat");

plot_correlation_scatter(data, formula, "fimo+.h3k4me3.scatter");

#
# 3. Run ChIP-seq (R-squared bootstrap, SVD, coeff)
#

formula <- esc ~  E2f1 + Esrrb + Klf4 + Nanog + Oct4 + Smad1 + Sox2 + Stat3 + Tcfcp2l1 + Zfx + Myc + Mycn;
svd_formula <- esc ~ PC1 + PC2 + PC3 + PC4 + PC5 + PC6 + PC7 + PC8 + PC9 + PC10 + PC11 + PC12;

x <- build_dataset_ouyang();
subset <- x[subset.ind,];
remainder <- x[-subset.ind,];

sigma <- find_best_sigma(subset, formula);

# Prepare data for bootstrap
y <- log(remainder$esc + sigma);
x.log <- remainder[,3:14]; #rest not required as already done
#x.log[,3:14] <- normalize.quantiles(as.matrix(x.log[,3:14]));
#x.log <- log(x.log[,c(3:14)] + sigma);
data <- cbind(y, x.log);
colnames(data)[1] <- 'esc';

# Run bootstraps
results <- boot(data=data, formula=formula, statistic=bs_rsq, R=boot_times);

bci <- boot.ci(results, type="perc", conf=0.95, index=1);
rsqcount <- rsqcount + 1;
rsq[rsqcount] <- results$t0;
rsq.l[rsqcount] <- bci$percent[,4];
rsq.u[rsqcount] <- bci$percent[,5];
names(rsq)[rsqcount] <- "ChIP-seq TFAS";

results_tstat <- boot(data=data, formula=formula, statistic=bs_tstat, R=boot_times);
plot_bootstrap_barplot(results_tstat, "chipseq.nosvd.tstat");

data.svd <- svd_data(data);

svd_results_rsq <- boot(data=data.svd, statistic=bs_ind_rsq, R=boot_times);
svd_results_tstat <- boot(data=data.svd, formula=svd_formula, statistic=bs_tstat, R=boot_times);

plot_bootstrap_barplot(svd_results_tstat, "chipseq.svd.tstat");
plot_svd_bootstrap_barplot(svd_results_tstat, svd_results_rsq, results_tstat, "chipseq.svd.tstat");
plot_correlation_scatter(data, formula, "chipseq.scatter");

#
# 4. Run Histone-TSS only (R-squared bootstrap, SVD, coeff)
#

formula <- esc ~ DNAse + H3K20me3 + H3K27me3 + H3K36me3 + H3K4me1 + H3K4me2 + H3K4me3 + H3K9me3;

svd_formula <- esc ~ PC1 + PC2 + PC3 + PC4 + PC5 + PC6 + PC7 + PC8;

x <- build_dataset('ensembl.fimoplus.loglikelihood.-30000to+30000.tsv');
subset <- x[subset.ind,];
remainder <- x[-subset.ind,];

sigma <- find_best_sigma(subset, formula);

# Prepare data for bootstrap
y <- log(remainder$esc + sigma);
x.log <- remainder;
x.log[,3:14] <- normalize.quantiles(as.matrix(x.log[,3:14]));
x.log[,21:28] <- normalize.quantiles(as.matrix(x.log[,21:28]));
x.log <- log(x.log[,c(21:28)] + sigma);
data <- cbind(y, x.log);
colnames(data)[1] <- 'esc';


results <- boot(data=data, formula=formula, statistic=bs_rsq, R=boot_times);

bci <- boot.ci(results, type="perc", conf=0.95, index=1);
rsqcount <- rsqcount + 1;
rsq[rsqcount] <- results$t0;
rsq.l[rsqcount] <- bci$percent[,4];
rsq.u[rsqcount] <- bci$percent[,5];
names(rsq)[rsqcount] <- "HAS";

results_tstat <- boot(data=data, formula=formula, statistic=bs_tstat, R=boot_times);
plot_bootstrap_barplot(results_tstat, "histone.nosvd.tstat");

data.svd <- svd_data(data);

svd_results_rsq <- boot(data=data.svd, statistic=bs_ind_rsq, R=boot_times);
svd_results_tstat <- boot(data=data.svd, formula=svd_formula, statistic=bs_tstat, R=boot_times);

plot_bootstrap_barplot(svd_results_tstat, "histone.svd.tstat");
plot_svd_bootstrap_barplot(svd_results_tstat, svd_results_rsq, results_tstat, "histone.svd.tstat");

plot_correlation_scatter(data, formula, "histone.scatter");

#
# 5. Run FIMO + H3K4me3 w/ Histone-TSS (R-squared bootstrap, SVD, coeff)
#
formula <- esc ~  E2f1 + Esrrb + Klf4 + Nanog + Oct4 + Smad1 + Sox2 + Stat3 + Tcfcp2l1 + Zfx + c.Myc + n.Myc +
		  DNAse + H3K20me3 + H3K27me3 + H3K36me3 + H3K4me1 + H3K4me2 + H3K4me3 + H3K9me3;

svd_formula <- esc ~ PC1 + PC2 + PC3 + PC4 + PC5 + PC6 + PC7 + PC8 + PC9 + PC10 + PC11 + PC12 +
		     PC13 + PC14 + PC15 + PC16 + PC17 + PC18 + PC19 + PC20;

x <- build_dataset('ensembl.fimoplus.loglikelihood.-30000to+30000.tsv');
subset <- x[subset.ind,];
remainder <- x[-subset.ind,];

sigma <- find_best_sigma(subset, formula);

# Prepare data for bootstrap
y <- log(remainder$esc + sigma);
x.log <- remainder;
x.log[,3:14] <- normalize.quantiles(as.matrix(x.log[,3:14]));
x.log[,21:28] <- normalize.quantiles(as.matrix(x.log[,21:28]));
x.log <- log(x.log[,c(3:14, 21:28)] + sigma);
data <- cbind(y, x.log);
colnames(data)[1] <- 'esc';


results <- boot(data=data, formula=formula, statistic=bs_rsq, R=boot_times);

bci <- boot.ci(results, type="perc", conf=0.95, index=1);
rsqcount <- rsqcount + 1;
rsq[rsqcount] <- results$t0;
rsq.l[rsqcount] <- bci$percent[,4];
rsq.u[rsqcount] <- bci$percent[,5];
names(rsq)[rsqcount] <- "FIMO+\nw/ HAS";

results_tstat <- boot(data=data, formula=formula, statistic=bs_tstat, R=boot_times);
plot_bootstrap_barplot(results_tstat, "fimo+.h3k4me3.histone.nosvd.tstat");

data.svd <- svd_data(data);

svd_results_rsq <- boot(data=data.svd, statistic=bs_ind_rsq, R=boot_times);
svd_results_tstat <- boot(data=data.svd, formula=svd_formula, statistic=bs_tstat, R=boot_times);

plot_bootstrap_barplot(svd_results_tstat, "fimo+.h3k4me3.histone.svd.tstat");
plot_svd_bootstrap_barplot(svd_results_tstat, svd_results_rsq, results_tstat, "fimo+.h3k4me3.histone.svd.tstat");
plot_correlation_scatter(data, formula, "fimo+.h3k4me3.histone.scatter");

#
# 6. Run ChIP-seq w/ Histone-TSS (R-squared bootstrap, SVD, coeff)
#

formula <- esc ~  E2f1 + Esrrb + Klf4 + Nanog + Oct4 + Smad1 + Sox2 + Stat3 + Tcfcp2l1 + Zfx + Myc + Mycn +
		  DNAse + H3K20me3 + H3K27me3 + H3K36me3 + H3K4me1 + H3K4me2 + H3K4me3 + H3K9me3;

svd_formula <- esc ~ PC1 + PC2 + PC3 + PC4 + PC5 + PC6 + PC7 + PC8 + PC9 + PC10 + PC11 + PC12 +
		     PC13 + PC14 + PC15 + PC16 + PC17 + PC18 + PC19 + PC20;

x <- build_dataset_ouyang();
subset <- x[subset.ind,];
remainder <- x[-subset.ind,];

sigma <- find_best_sigma(subset, formula);

# Prepare data for bootstrap
y <- log(remainder$esc + sigma);
x.log <- remainder[,c(3:14,21:28)]; #rest not required as already done
x.log[,13:20] <- normalize.quantiles(as.matrix(x.log[,13:20]));
x.log[,13:20] <- log(x.log[,c(13:20)] + sigma);
data <- cbind(y, x.log);
colnames(data)[1] <- 'esc';

# Run bootstraps
results <- boot(data=data, formula=formula, statistic=bs_rsq, R=boot_times);

bci <- boot.ci(results, type="perc", conf=0.95, index=1);
rsqcount <- rsqcount + 1;
rsq[rsqcount] <- results$t0;
rsq.l[rsqcount] <- bci$percent[,4];
rsq.u[rsqcount] <- bci$percent[,5];
names(rsq)[rsqcount] <- "ChIP-seq TFAS\nw/ HAS";

results_tstat <- boot(data=data, formula=formula, statistic=bs_tstat, R=boot_times);
plot_bootstrap_barplot(results_tstat, "chipseq.histone.nosvd.tstat");

data.svd <- svd_data(data);

svd_results_rsq <- boot(data=data.svd, statistic=bs_ind_rsq, R=boot_times);
svd_results_tstat <- boot(data=data.svd, formula=svd_formula, statistic=bs_tstat, R=boot_times);

plot_bootstrap_barplot(svd_results_tstat, "chipseq.histone.svd.tstat");
plot_svd_bootstrap_barplot(svd_results_tstat, svd_results_rsq, results_tstat, "chipseq.histone.svd.tstat");
plot_correlation_scatter(data, formula, "chipseq.histone.scatter");

######################
# 7. Plot overall r^2
######################
save.image("regressiondata.rData");

pdf("rsqcomparison.pdf");
par(cex.axis=1.3, cex.lab=1.3, cex.main=1.3,mar=c(10,4,1,1) + 0.1); 
cols <- rainbow(length(rsq));
barplot2(rsq, col=cols, main='', las=2, ylab="Adjusted R-squared", 
         plot.ci=TRUE, ci.l=rsq.l, ci.u=rsq.u);
dev.off();
