library(preprocessCore);
library(gplots);
library(boot);
library(mixtools);

get_cpg_cutoff <- function(data) {
	cpg <- data$CpGRatio;
	mixmdl <- normalmixEM(cpg);
	# Find the split point (where class probabilities are 0.5 each).
	return(mixmdl$x[which(abs(mixmdl$posterior-0.5) == min(abs(mixmdl$posterior-0.5)))[1]]);
}

bs_rsq <- function(data, formula, indices) {
  d <- data[indices, ];
	m <- lm(formula, data=d);
	return(summary(m)$adj.r.squared);
}

# T-statistics, actually
bs_tstat <- function(data, formula, indices) {
	d <- data[indices, ];
	m <- lm(formula, data=d);
	return(summary(m)$coefficients[2:nrow(summary(m)$coefficients),3]);
}

bs_svd_rsq <- function(data, formula, indices) {
	d <- data[indices, ];

	d.svd <- svd(d[,2:length(d)]); #Don't SVD the expression column
	d[,2:length(d)] <- d.svd$u %*% diag(d.svd$d);
	colnames(d)[2:length(d)] <- paste("PC", c(1:(length(d)-1)),sep='');

	m <- lm(formula, data=d);
	return(summary(m)$adj.r.squared);
}

# T-statistics, actually
bs_svd_tstat <- function(data, formula, indices) {
	d <- data[indices, ];

	d.svd <- svd(d[,2:length(d)]); #Don't SVD the expression column
	d[,2:length(d)] <- d.svd$u %*% diag(d.svd$d);
	colnames(d)[2:length(d)] <- paste("PC", c(1:(length(d)-1)),sep='');

	m <- lm(formula, data=d);
	return(summary(m)$coefficients[2:nrow(summary(m)$coefficients),3]);
}

bs_ind_rsq <- function(data.svd, indices) {
	d <- data.svd[indices, ];
	y <- data.svd[indices, "esc"];
	# Build R^2 table
	rsq.tfpc <- vector();
	for (i in 2:length(d)) {
		m <- lm(y ~ d[,i]);
		rsq.tfpc[i-1] <- summary(m)$r.squared;
	}
	names(rsq.tfpc) <- paste("PC", c(1:(length(d)-1)),sep='');
	return(rsq.tfpc)
}

svd_data <- function(d) {
  d.svd <- svd(d[,2:length(d)]); #Don't SVD the expression column
  d[,2:length(d)] <- d.svd$u %*% diag(d.svd$d);
  colnames(d)[2:length(d)] <- paste("PC", c(1:(length(d)-1)),sep='');
  return(d);
}

build_dataset <- function(genedatafile, histonefile='histone_promoter_map.ensembl.4000.4000.tsv') {

	genedata <- read.table(genedatafile, header=T);
	expdata <- read.table('unique_mappers_entrez_to_ensembl.tsv');
	histonedata <- read.table(histonefile, header=T);
	genes_to_keep <- read.table('Entrez_IDs_To_Keep_in_Ouyang.tsv');

	colnames(genedata)[1] <- "Ensembl.ID";
	colnames(expdata)[1] <- "Entrez_ID";
	colnames(histonedata)[1] <- "Ensembl.ID";
	colnames(genes_to_keep)[1] <- 'Input';

	x<-merge(genedata,expdata,by="Ensembl.ID");
	x<-merge(x,histonedata,by="Ensembl.ID");
	x<-merge(x,genes_to_keep,by="Input");
  
	# Remove 39 duplicates caused by MGI data crud
	x <- x[!duplicated(x[,"Entrez_ID"]),]

	# Fix TF name
	colnames(x)[8] <- "Smad1";

	# Simple assertion
 	print(paste("nrow(x)", nrow(x), "nrow(genes_to_keep)", nrow(genes_to_keep)));
	stopifnot(nrow(x) == nrow(genes_to_keep));

	return(x);
}

build_dataset_ouyang <- function(histonefile='histone_promoter_map.ensembl.4000.4000.tsv') {

	mdata <- read.table('data_from_ouyang/SD2.txt', header=T);
	fdata <- read.table('unique_mappers_entrez_to_ensembl.tsv')
	hdata <- read.table(histonefile, header=T);
	genes_to_keep <-read.table('Entrez_IDs_To_Keep_in_Ouyang.tsv')

	colnames(fdata)[1] <- "Entrez_ID";
	colnames(hdata)[1] <- "Ensembl.ID";
	colnames(genes_to_keep)[1] <- 'Entrez_ID';

	x<-merge(mdata,fdata,by="Entrez_ID");
	x<-merge(x,hdata,by="Ensembl.ID");
	x<-merge(x,genes_to_keep,by="Entrez_ID");

	# Remove 39 duplicates caused by MGI data crud
	x <- x[!duplicated(x[,"Entrez_ID"]),]

	# Re-jigger into same column order as above
	tforder <- c(4, 10, 8, 11, 12, 15, 13, 14, 9, 6, 7, 5);
	x <- x[,c(19, 2, tforder, 1, 16:18, 20:length(x))]
	
	# Simple assertion 
	stopifnot(nrow(x) == nrow(genes_to_keep));

	return(x);

}

find_best_sigma <- function(data, formula) {
	rsq.df <- c(0,0);
	for (power in -10:10) {
		x.log <- subset;
		x.log[,3:14] <- normalize.quantiles(as.matrix(x.log[,3:14]));
		x.log[,21:length(x.log)] <- normalize.quantiles(as.matrix(x.log[,21:length(x.log)]));
		x.log <- log(x.log[,c(3:14,21:length(x.log))] + 10^power);
		x.log$esc <- log(data$esc+10^power);
		m <- lm(formula, data=x.log);
		rsq.df <- rbind(rsq.df, c(10^power, summary(m)$r.squared));
	}
	return(as.numeric(rsq.df[which(max(rsq.df[,2]) == rsq.df[,2]),1]));
}

plot_bootstrap_barplot <- function(bootdata, filename) {
	# Get 95% intervals
	ci.l <- array();
	ci.u <- array();
	regression_coef <- array();

	for (i in 1:length(bootdata$t0)) {
	  bci <- boot.ci(bootdata, type="perc", conf=0.95, index=i);
	  regression_coef[i] <- bci$t0;
	  ci.l[i] <- bci$percent[,4];
	  ci.u[i] <- bci$percent[,5];
	}
	names(regression_coef) <- names(bootdata$t0);

	# Plot the barchart of the regression co-efficients
	cols <- rainbow(length(bootdata$t0));

	# Reorder
	o <- order(regression_coef, decreasing=TRUE);
	regression_coef <- regression_coef[o];
	ci.l <- ci.l[o];
	ci.u <- ci.u[o];
	sortcols <- cols[o];
	
	pdf(paste(filename, ".pdf", sep=''));
	par(cex.axis=1.3, cex.lab=1.3, cex.main=1.3,mar=par()$mar+c(2,0,-2,0));
	barplot2(regression_coef, col=sortcols, main='', las=2, ylab="t-statistic", plot.ci=TRUE, ci.l=ci.l, ci.u=ci.u);
	dev.off();
	
	save(bootdata, file=paste(filename,".rData",sep=''));

}

plot_svd_bootstrap_barplot <- function(tstatbootdata, svdbootdata, nosvdbootdata, filename) {
	# Get 95% intervals
	ci.l <- array();
	ci.u <- array();
	regression_coef <- array();

	for (i in 1:length(tstatbootdata$t0)) {
	  bci <- boot.ci(tstatbootdata, type="perc", conf=0.95, index=i);
	  regression_coef[i] <- bci$t0;
	  ci.l[i] <- bci$percent[,4];
	  ci.u[i] <- bci$percent[,5];
	}
	names(regression_coef) <- names(tstatbootdata$t0);


	rsq <- array();
	rsq.ci.l <- array();
	rsq.ci.u <- array();

	# Extract the confidence intervals
	for (i in 1:length(svdbootdata$t0)) {
		bci <- boot.ci(svdbootdata, type="perc", conf=0.95, index=i);
		rsq[i] <- bci$t0;
		rsq.ci.l[i] <- bci$percent[,4];
		rsq.ci.u[i] <- bci$percent[,5];
	}

	# Do svd manually here to get the v array.
	data.svd <- svd(nosvdbootdata$data[,2:length(nosvdbootdata$data)]); #Don't SVD the expression column


	for (i in 1:length(svdbootdata$t0)) {

		if (i == 1) {
			pcylim <- c(0, 0.1);
		} else if (i == 2) {
			pcylim <- c(-0.15, 0.15);
		} else {
			pcylim <- c(-0.1, 0.1);
		}

		if (regression_coef[i] < 0) {
			switchsign <- -1;
		} else {
			switchsign <- 1;
		}
		p <- data.svd$v[,i]*rsq[i]*switchsign; #multiply by TFPC r^2.

		# TODO fix the code below
		p.ci.u <- data.svd$v[,i]*rsq.ci.u[i]*switchsign;
		p.ci.l <- data.svd$v[,i]*rsq.ci.l[i]*switchsign;
		names(p) <- colnames(nosvdbootdata$data)[2:length(nosvdbootdata$data)];
		cols <- rainbow(length(tstatbootdata$data) - 1);
		#sortcols <- cols[tforder];
		#p.ci.u <- p.ci.u[tforder];
		#p.ci.l <- p.ci.l[tforder];
		#p <- p[tforder];

		pdf(paste(filename, ".TFPC", i, ".pdf", sep=''));
		par(cex.axis=1.3, cex.lab=1.3, cex.main=1.3,mar=par()$mar+c(3,0,-2,0));
		barplot2(p, col=cols, main='', las=2, ylab="Weighted Loading", 
			 plot.ci=TRUE, ci.l=p.ci.l, ci.u=p.ci.u, ylim=pcylim);
		dev.off();
	}
	save(tstatbootdata, svdbootdata, nosvdbootdata,file=paste(filename,".components.rData",sep=''));
}

plot_correlation_scatter <- function(data, formula, filename) {
	m <- lm(formula, data=data);
	rval <- round(cor.test(data$esc, (data$esc - m$residuals))$estimate,3)

	pdf(paste(filename, ".pdf", sep=''));
	par(cex.axis=1.3, cex.lab=1.3, cex.main=1.3,mar=par()$mar+c(0,0,-2,0));
	plot(data$esc, (data$esc - m$residuals), xlab="log(RNA-seq ESC Expression)", ylab="log(Predicted Expression)");
	legend("bottomright", legend=bquote(italic(r) == .(rval)), bty='n', cex=1.3);
	dev.off();
	save(data,formula,file=paste(filename,".rData",sep=''));
}
